require 'uu/os/http/http_headers'

module UU
  module OS
    module HTTP

      # HTTP response.
      class HTTPResponse

        # Creates new instance of HTTP response.
        # @private
        def initialize(response, request = nil) # :nodoc:
          @response = response
          @request = request
          @headers = UU::OS::HTTP::HTTPHeaders.new
        end

        # Returns response HTTP status code. In case of asynchronous
        # request calling of this method causes wait for request finish.
        # @return [Fixnum] HTTP status code
        def status
          process_response
          return @response.status
        end

        # Returns response headers. In case of asynchronous request
        # calling of this method causes wait for request finish.
        # @return [UU::OS::HTTP::HTTPHeaders] Response headers.
        def headers
          process_response
          return @headers
        end

        # Returns response body. In case of asynchronous request
        # calling of this method causes wait for request finish.
        # @return [String, IO] IO is returned for asynchronous request, else String is returned.
        def body
          process_response
          return @response.body
        end

        # Returns request which resulted in this response. In case
        # of asynchronous request calling of this method causes wait
        # for request finish, so that is not possible to modify request
        # before it is processed.
        # Note that if request body is IO it will be returned in consumed
        # state and in case it does not support +rewind+ operation there
        # is no way to obtain original request data.
        # @return [UU::OS::HTTP::HTTPRequest] HTTP request
        def request
          process_response
          return @request
        end

        # Override inspect to hide internal structures.
        # @private
        def inspect # :nodoc:
          # For asynchronous request, inspect must not start processing of result.
          # Therefore we cannot display anything useful in here.
          "#<#{self.class.name}:0x#{(self.object_id << 1).to_s(16)}>"
        end

        private

        # Processes response (waits for asynchronous request if necessary
        # and parses response headers if not yet parsed).
        # @private
        def process_response # :nodoc:
          if @response.kind_of?(::HTTPClient::Connection)
            @response = @response.pop
            raise @response if @response.is_a?Exception
          end
          if @headers.empty?
            @response.header.all.each do |header|
              @headers.add(header[0], header[1])
            end
          end
        end

      end

    end
  end
end
